searchData={"items":[{"type":"module","title":"Peeper","doc":"`Peeper.GenServer` is an almost drop-in replacement for `GenServer` that preserves the state\n  between crashes. All the callbacks from `GenServer` are supported.\n\nInternally it creates a sub-supervision tree with an actual worker that delegates to the\n  implementation and a state keeper. That said, it creates three processes instead of one,\n  and this should be considered when building a very concurrent applications.\n\nThe main use-case would be a long-lived `GenServer` process with no error path handling at all\n  (yeah, that famous _Let It Crash_.) Using this abstraction, one is free to send unexpected\n  messages to the process, raise from its `handle_×××` clauses and whatnot.\n\nThere are two differencies compared to bare `GenServer`. `init/1` callback cannot return anything\n  but `{:ok, state}` or `{:ok, new_state, timeout() | :hibernate | {:continue, term()}` tuples\n  (this might have changed in the future,) and `Peeper.{call/3,cast/2,send/2}` is to be used\n  instead of `GenServer.{call/3,cast/2}` and `Kernel.send/2` (this is not gonna change.)\n\nPlease note, that whatever is set in `init/1` callback as a state, will be overriden by\n  the latest state available upon restarts. That being said, `init/1` would play its role\n  in setting the state during the first run only. \n\nInstead of using `Peeper`’s counterparts, one might either name the process and use `Name.GenServer`\n  as a name of the underlying `GenServer` or get the `GenServer`’s `pid` via `Peeper.gen_server/1`\n  and use `GenServer.{call/3,cast/2}` with it.","ref":"Peeper.html"},{"type":"module","title":"Example - Peeper","doc":"iex> {:ok, pid} = Peeper.Impls.Full.start_link(state: 0, name: Counter)\n    ...> Peeper.call(pid, :state)\n    0\n    iex> Peeper.cast(pid, :inc)\n    :ok\n    iex> GenServer.call(Peeper.gen_server(Counter), :state)\n    1\n    iex> Peeper.call(pid, :state)\n    1\n    iex> # emulate crash\n    ...> Process.exit(Peeper.Supervisor.worker(pid), :raise)\n    ...> %{} = Peeper.Supervisor.which_children(pid)\n    iex> Peeper.call(Counter, :state)\n    1\n    iex> Peeper.send(pid, :inc)\n    :inc\n    iex> Peeper.call(Counter, :state)\n    2\n\n### `start_link/1`\n\nThe function receives either an initial `state`, or a keyword having keys\n  `state` and (optionally) `name`. Also this keyword might have a configuration\n  for the top supervisor (keys: `[:strategy, :max_restarts, :max_seconds, :auto_shutdown]`.)\n\nAll other values passed would be re-passed to the underlying `GenServer`’s options.","ref":"Peeper.html#module-example"},{"type":"module","title":"Keeping ETS - Peeper","doc":"`Peeper` can preserve ETS tables created by the wrapped process between crashes in several ways.\n\nThe proper solution would be to add a `:heir` option to the ETS created by the process as\n\n```elixir\n:ets.new(name, [:private, :ordered_set, Peeper.heir(MyNamedPeeper)])\n```\n\nThat way the ETS will remain private and not readable by any other part of the system, although\n  it’ll be preserved between crashes _and_ might be transferred to another dynamic supervisor (see below.)\n\nIf for some reason setting `:heir` explicitly is not an option, one might pass \n  `keep_ets: true | :all | [ets_name()]` option to `Peeper.start_link/1`. It’s not efficient, because\n  ETS content will be passed tyo the backing state process every time the change to it happens to occur.\n\nIf the ETS created by the underlined process has `{:heir, other_process_pid()}` set, ithe behaviour\n  after a process crash is undefined, because ETS will be transferred to another process and reach out\n  of the control of `Peeper`.","ref":"Peeper.html#module-keeping-ets"},{"type":"module","title":"Moving to another `DynamicSupervior` - Peeper","doc":"The `Peeper` branch might be transferred an a whole to another dynamic supervisor in the following way\n\n```elixir\nPeeper.transfer(MyNamedPeeper, source_dynamic_supervisor, target_dynamic_supervisor)\n```\n\nThe `target_dynamic_supervisor` might be a remote `pid`, in such a case `Peeper` must be compiled \n  and loaded on the target node.","ref":"Peeper.html#module-moving-to-another-dynamicsupervior"},{"type":"module","title":"Listener - Peeper","doc":"One might pass `listener: MyListener` key to `PeeperImpl.start_link/1` where `MyListener`\n  should be an implementation of `Peeper.Listener` behaviour. The callbacks will be called\n  when the state of the underlying behaviour is _changed_ and on termination respectively.\n\nThat might be a good place to attach telemetry events, or logs, or whatnot.","ref":"Peeper.html#module-listener"},{"type":"function","title":"Peeper.call/3","doc":"The counterpart for `GenServer.call/3`. Uses the very same semantics.","ref":"Peeper.html#call/3"},{"type":"function","title":"Peeper.cast/3","doc":"The counterpart for `GenServer.cast/2`. Uses the very same semantics.","ref":"Peeper.html#cast/3"},{"type":"function","title":"Peeper.child_spec/1","doc":"Returns a specification to start a branch under a supervisor.","ref":"Peeper.html#child_spec/1"},{"type":"type","title":"Peeper.ets_option/0","doc":"The option accepted by `:ets.new/2`","ref":"Peeper.html#t:ets_option/0"},{"type":"function","title":"Peeper.gen_server/2","doc":"Returns the `pid` of the actual `GenServer`. Might be used to\navoid the necessity of calling other functions in this module.\n\nThe `pid` returned might be used directly in calls to\n`GenServer.{call/3,cast/2}` and/or `Kernel.send/2`","ref":"Peeper.html#gen_server/2"},{"type":"function","title":"Peeper.gen_server_name/1","doc":"Tries to produce a name for the underlying `GenServer` process.\nSucceeds if the name passed to `start_link/1` of the implementation\nis an `atom()`, `{:global, atom()}`, or `{:via, module(), atom()}`.\n\nIn such a case, the underlying `GenServer` module receives the name\n`Module.concat(name, GenServer)` and might be used as such.","ref":"Peeper.html#gen_server_name/1"},{"type":"function","title":"Peeper.heir/2","doc":"Returns a `{:heir, pid(), heir_data}` tuple where `pid` is the\npid of the state holder. Useful when the `GenServer`\nprocess creates persistent _ETS_, the result of this function is\nto be passed to _ETS_ options as a config parameter.\n\nIn that case, the tables will be given away to the state process\nand then retaken by the restarted `GenServer`.","ref":"Peeper.html#heir/2"},{"type":"type","title":"Peeper.name/0","doc":"The name of the processes branch","ref":"Peeper.html#t:name/0"},{"type":"function","title":"Peeper.send/3","doc":"The counterpart for `Kernel.send/2`. Uses the very same semantics.","ref":"Peeper.html#send/3"},{"type":"function","title":"Peeper.start_link/1","doc":"Starts a supervisor branch with the given options.","ref":"Peeper.html#start_link/1"},{"type":"function","title":"Peeper.transfer/4","doc":"Transfers the whole `Peeper` branch from one supervisor to another.","ref":"Peeper.html#transfer/4"},{"type":"behaviour","title":"Peeper.GenServer","doc":"A drop-in replacement for `use GenServer`","ref":"Peeper.GenServer.html"},{"type":"macro","title":"Peeper.GenServer.__using__/1","doc":"Declares a `Peeper.GenServer` behaviour, injects `start_link/1` function\nand the child spec.","ref":"Peeper.GenServer.html#__using__/1"},{"type":"macro","title":"Example - Peeper.GenServer.__using__/1","doc":"```elixir\ndefmodule MyGenServer do\n  use Peeper.GenServer\n\n  @impl Peeper.GenServer\n  def handle_call(:state, _from, state),\n    do: {:reply, state, state}\n\n  @impl Peeper.GenServer\n  def handle_cast(:inc, state),\n    do: {:noreply, state, {:continue, :inc}}\n\n  @impl Peeper.GenServer\n  def handle_continue(:inc, state),\n    do: {:noreply, state + 1}\n  end\n```","ref":"Peeper.GenServer.html#__using__/1-example"},{"type":"callback","title":"Peeper.GenServer.code_change/3","doc":"_See:_ `c:GenServer.code_change/3`","ref":"Peeper.GenServer.html#c:code_change/3"},{"type":"callback","title":"Peeper.GenServer.handle_call/3","doc":"_See:_ `c:GenServer.handle_call/3`","ref":"Peeper.GenServer.html#c:handle_call/3"},{"type":"callback","title":"Peeper.GenServer.handle_cast/2","doc":"_See:_ `c:GenServer.handle_cast/2`","ref":"Peeper.GenServer.html#c:handle_cast/2"},{"type":"callback","title":"Peeper.GenServer.handle_continue/2","doc":"_See:_ `c:GenServer.handle_continue/2`","ref":"Peeper.GenServer.html#c:handle_continue/2"},{"type":"callback","title":"Peeper.GenServer.handle_info/2","doc":"_See:_ `c:GenServer.handle_info/2`","ref":"Peeper.GenServer.html#c:handle_info/2"},{"type":"callback","title":"Peeper.GenServer.init/1","doc":"Almost the same as `c:GenServer.init/1` but cannot return anything but `{:ok, new_state}`\n  tuple and is being invoked from `handle_continue/2` followed `init/1` of the state\n  keeper process.\n\n_See:_ `c:GenServer.init/1`","ref":"Peeper.GenServer.html#c:init/1"},{"type":"function","title":"Peeper.GenServer.start_link/2","doc":"Starts a `Peeper` sub-supervision process tree linked to the current process.","ref":"Peeper.GenServer.html#start_link/2"},{"type":"callback","title":"Peeper.GenServer.terminate/2","doc":"_See:_ `c:GenServer.terminate/2`","ref":"Peeper.GenServer.html#c:terminate/2"},{"type":"behaviour","title":"Peeper.Listener","doc":"The listener that might be implemented and attached to the `Peeper.GenServer`\n  to receive notifications about process state changes and/or terminations.","ref":"Peeper.Listener.html"},{"type":"callback","title":"Peeper.Listener.on_state_changed/2","doc":"Called when the underlying `Peeper.GenServer`’s state gets changed","ref":"Peeper.Listener.html#c:on_state_changed/2"},{"type":"callback","title":"Peeper.Listener.on_terminate/2","doc":"Called when the underlying `Peeper.GenServer` gets terminated","ref":"Peeper.Listener.html#c:on_terminate/2"},{"type":"extras","title":"Peeper","doc":"# Peeper\n\n**Almost drop-in replacement for `GenServer` to preserve state between crashes.**","ref":"readme.html"},{"type":"extras","title":"Benefits - Peeper","doc":"`Peeper.GenServer` is an almost drop-in replacement for `GenServer` that preserves the state\n  between crashes. All the callbacks from `GenServer` are supported.\n\nInternally it creates a sub-supervision tree with an actual worker that delegates to the\n  implementation and a state keeper. That said, it creates three processes instead of one,\n  and this should be considered when building a very concurrent applications.\n\nThe main use-case would be a long-lived `GenServer` process with no error path handling at all\n  (yeah, that famous _Let It Crash_.) Using this abstraction, one is free to send unexpected\n  messages to the process, raise from its `handle_×××` clauses and whatnot.\n\nThere are two differencies compared to bare `GenServer`. `init/1` callback cannot return anything\n  but `{:ok, state}` tuple (this might have changed in the future,) and\n  `Peeper.{call/3,cast/2,send/2}` is to be used instead of `GenServer.{call/3,cast/2}` and\n  `Kernel.send/2` (this is not gonna change.)\n\nPlease note, that whatever is set in `init/1` callback as a state, will be overriden by\n  the latest state available upon restarts. That being said, `init/1` would play its role\n  in setting the state during the first run only. \n\nInstead of using `Peeper`’s counterparts, one might either name the process and use `Name.GenServer`\n  as a name of the underlying `GenServer` or get the `GenServer`’s `pid` via `Peeper.gen_server/1`\n  and use `GenServer.{call/3,cast/2}` with it.","ref":"readme.html#benefits"},{"type":"extras","title":"Example - Peeper","doc":"```elixir\n    iex> {:ok, pid} = Peeper.Impls.Full.start_link(state: 0, name: Counter)\n    ...> Peeper.call(pid, :state)\n    0\n    iex> Peeper.cast(pid, :inc)\n    :ok\n    iex> GenServer.call(Counter.GenServer, :state)\n    1\n    iex> Peeper.call(pid, :state)\n    1\n    iex> # emulate crash\n    ...> Process.exit(Peeper.Supervisor.worker(pid), :raise)\n    ...> %{} = Peeper.Supervisor.which_children(pid)\n    iex> Peeper.call(Counter, :state)\n    1\n    iex> Peeper.send(pid, :inc)\n    :inc\n    iex> Peeper.call(Counter, :state)\n    2\n```\n\n### `start_link/1`\n\nThe function receives either an initial `state`, or a keyword having keys\n  `state` and (optionally) `name`. Also this keyword might have a configuration\n  for the top supervisor (keys: `[:strategy, :max_restarts, :max_seconds, :auto_shutdown]`.)\n\nAll other values passed would be re-passed to the underlying `GenServer`’s options.","ref":"readme.html#example"},{"type":"extras","title":"Listener - Peeper","doc":"One might pass `listener: MyListener` key to `PeeperImpl.start_link/1` where `MyListener`\n  should be an implementation of `Peeper.Listener` behaviour. The callbacks will be called\n  when the state of the underlying behaviour is _changed_ and on termination respectively.\n\nThat might be a good place to attach telemetry events, or logs, or whatnot.","ref":"readme.html#listener"},{"type":"extras","title":"Usage - Peeper","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Installation - Peeper","doc":"```elixir\ndef deps do\n  [\n    {:peeper, \"~> 0.1\"}\n  ]\nend\n```\n\n### [Documentation](https://hexdocs.pm/peeper)","ref":"readme.html#installation"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}